#!/System/Library/Frameworks/Ruby.framework/Versions/Current/usr/bin/ruby

#  popmaker.rb
#  PopMaker
#
#  v1.0 Created by Brett Terpstra on 5/6/14.
#  Copyright (c) 2014 Brett Terpstra. All rights reserved.

require 'fileutils'
require 'cgi'

class PopMaker

	SEPARATOR = "@@!!@@"

	def initialize(user,extname,title,has_icon,should_zip,type,payload)

		username = user.gsub(/[^a-z]/i,'').downcase;
		filename = extname.gsub(/[^a-z0-9]/i,'')
		if filename == ""
			filename = "#{username}-#{rand(10000)}"
		end
		req = false
		after = false
		script_template = false

		if type == "surround"
			data = payload.split(SEPARATOR)
			prefix = data[0]
			suffix = data[1]
			script_template = surround_script(prefix, suffix)
			req = "paste"
			after = "paste-result"
			elsif type == "search"
			script_template = search_script(payload)
		end

		unless script_template
			print "ERROR"
			Process.exit 1
		end

		plist = plist_template(title, parse_requirements(req), parse_after(after), extname, user, username, filename, has_icon)

		if ENV.has_key? "CONTAINER_DIR"
			tmpdir = File.join(ENV["CONTAINER_DIR"],"Library/Application Support/PopMaker")
			unless File.exists? tmpdir
				FileUtils.mkdir_p(tmpdir)
			end
			else
			tmpdir = ENV["TMPDIR"]
		end

		extension_location = File.join(tmpdir,filename+".popclipext")
		FileUtils.mkdir_p(extension_location)

		fh = File.new(File.join(extension_location, "script.rb"),'w+')
		fh.puts script_template
		fh.close

		fh = File.new(File.join(extension_location, "Config.plist"),'w+')
		fh.puts plist
		fh.close

		if has_icon and has_icon != ""
			if File.exists?(has_icon)
				FileUtils.cp(has_icon, File.join(extension_location, "icon.png"))
			end
		end

		if should_zip
			extension_file = File.join(tmpdir,filename + ".popclipextz")
			FileUtils.rm(extension_file) if File.exists?("#{extension_file}")
			system "cd '#{tmpdir}' && zip '#{extension_file}' #{filename}.popclipext/* 1> /dev/null"
			FileUtils.rmtree(extension_location)
			else
			extension_file = extension_location
		end

		if File.exists?("#{extension_file}")
			# FileUtils.rmtree(extension_location)
			print extension_file
			Process.exit 0
		else
			print "ERROR"
			Process.exit 1
		end

	end

	def parse_requirements(req)
		return "" unless req
		out=<<-EOS
		<key>Requirements</key>
		<array>
		<string>#{req}</string>
		</array>
		EOS
		return out
	end

	def parse_after(after)
		return "" unless after
		out =<<-EOS
		<key>After</key>
		<string>#{after}</string>
		EOS
		return out
	end

	def surround_script(prefix,suffix)
		out =<<-EOS
		#!/System/Library/Frameworks/Ruby.framework/Versions/Current/usr/bin/ruby

		prefix = "#{prefix}"
		suffix = "#{suffix}"
		input = ENV['POPCLIP_TEXT']

		space = input.match(/^([\\s\\n]*)\\S.*?([\\s\\n]*)$/m)
		print "\#{space[1]}\#{prefix}\#{input.strip}\#{suffix}\#{space[2]}"
		EOS
		return out
	end

	def search_script(url)
		out =<<-EOS
		#!/System/Library/Frameworks/Ruby.framework/Versions/Current/usr/bin/ruby

		require 'cgi'

		url = "#{url}"
		query = ENV['POPCLIP_TEXT']
		url.sub!(/\\{query\\}/,CGI.escape(query))

		%x{open "\#{url}"}
		EOS
		return out
	end

	def plist_template(title, requirements, after, extname, user, username, filename, has_icon)
		if has_icon
			icon =<<-EOS
			<key>Image File</key>
			<string>icon.png</string>
			EOS
			else
			icon = ""
		end

		out =<<-EOS
		<?xml version="1.0" encoding="UTF-8"?>
		<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
		<plist version="1.0">
		<dict>
		<key>Actions</key>
		<array>
		<dict>
		<key>Script Interpreter</key>
		<string>/usr/bin/ruby</string>
		<key>Shell Script File</key>
		<string>script.rb</string>
		<key>Title</key>
		<string>#{title}</string>
		#{icon}#{after}#{requirements}
		</dict>
		</array>
		<key>Extension Description</key>
		<string>#{extname}, made with PopMaker by #{user}</string>
		<key>Extension Identifier</key>
		<string>com.#{username}.popclip.extension.#{filename}</string>
		<key>Extension Name</key>
		<string>#{extname}</string>
		<key>Version</key>
		<integer>2</integer>
		</dict>
		</plist>
		EOS
		return out
	end

end

user = ARGV[0]
extname = ARGV[1]
title = ARGV[2]
has_icon = ARGV[3] == "" ? false : ARGV[3]
should_zip = ARGV[4].to_i == 0 ? false : true
type = ARGV[5]
payload = ARGV[6]
PopMaker.new(user,extname,title,has_icon,should_zip,type,payload)